/* slld_hal.c - Hardware Abstraction Layer Code for Cyress's SLLD */
 
/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#include <stdio.h>
#include "slld_targetspecific.h"
#include "slld.h"
#include "slld_hal.h"
#include "NORFlash.h"

//***************************************************************************
//  FLASH_READ - HAL read function
//
//  input : device_num            device number to which operation will be done
//          command               write a single command byte to flash
//          sys_addr              system address to be used
//          data_buffer           Pointer to the data buffer where to store the read data
//          Number_Of_Read_Bytes  number of bytes to be read
//
//  return value : status of the operation - FAIL or SUCCESS
//***************************************************************************
SLLD_STATUS FLASH_READ
(
BYTE    device_num,                     /* device number to which operation will be done */
BYTE     command,                        /* write a single command byte to flash */
ADDRESS  sys_addr,                       /* system address to be used */
BYTE    *data_buffer,                    /* Pointer to the data buffer containing data to be written */
int      Number_Of_Read_Bytes            /* number of bytes to be read */
)
{
  SLLD_STATUS status = SLLD_OK;
  
  switch(command)
  {
  case SPI_RDSR_CMD:
    if(TRUE != NORFLASH_ReadStatusRegister(data_buffer))
    {
      status = SLLD_ERROR;
    }
    break;
  case SPI_RDSR2_CMD:
    if(TRUE != NORFLASH_ReadStatusRegister2(data_buffer))
    {
      status = SLLD_ERROR;
    }
    break;
  case SPI_READ_CMD:
    /* Need single SPI call only. */
    status = SLLD_ERROR;
    break;
  case SPI_QUADIO_RD_CMD:
    if(TRUE != NORFLASH_ReadChunkQuadIO(sys_addr,data_buffer,Number_Of_Read_Bytes))
    {
      status = SLLD_ERROR;
    }    
    break;
  }
  return(status);
}


//***************************************************************************
//  FLASH_WRITE - HAL write function
//
//  input : device_num               device number to which operation will be done
//          command                  write a single command byte to flash
//          sys_addr                 system address to be used
//          data_buffer              Pointer to the data buffer where to store the written data
//          Number_Of_Written_Bytes  number of bytes to be written
//
//  return value : status of the operation - FAIL or SUCCESS
//***************************************************************************
SLLD_STATUS FLASH_WRITE
(
BYTE    device_num,                     /* device number to which operation will be done */
BYTE     command,                        /* write a single command byte to flash */
ADDRESS  sys_addr,                       /* system address to be used */
BYTE    *data_buffer,                    /* Pointer to the data buffer containing data to be written */
int      Number_Of_Written_Bytes         /* number of bytes to be written */
)
{
  SLLD_STATUS status = SLLD_OK;
  
  //if (sys_addr != ADDRESS_NOT_USED)
  {
    switch (command)
    {
    case SPI_PP_CMD:
      {
        /* Need single SPI call only. */
        status = SLLD_ERROR;
        break;
      }
    case SPI_WREN_CMD:
      {
        if (TRUE!=NORFLASH_WriteEnableCMD())
        {
          status=SLLD_ERROR;
        }
        break;
      }
    case SPI_WRDI_CMD:
      if(TRUE!= NORFLASH_WriteDisableCMD())
      {
        status=SLLD_ERROR;
      }
      break;
    case SPI_SE_CMD:
      if(TRUE != NORFLASH_MemErase(sys_addr, SPI_SE_CMD))
      {
        status = SLLD_ERROR;
      }
      break;
    case  SPI_QPP_CMD: 
      if(TRUE != NORFLASH_WriteQuadMemory(sys_addr,data_buffer,Number_Of_Written_Bytes))
      {
        status = SLLD_ERROR;
      }
      break;
    default:
      {
        // Shift out the address one bit at a time in 3-bytes addressing scheme
        //printf("Single address write in 3-bytes addressing scheme, address: 0x%X, command: 0x%X\n", sys_addr, command);
        break;
      }
    }
  }
  return status;
}


/*****************************************************************************/
